c--------------------------------------------------------------------------
c  Program for reading hypoinverse archive format data and converting to 
c  nordic format.
c
c  All events are assumed local
c  No check if header time corresponds to phase times

c  Little tested
c  jh november 2011
c  
c jan 8 2013 jh: fix small format change, add posibility for arc line
c--------------------------------------------------------------------------c
c
c  For detail on parameters and variables names, see rea.inc
c
c
c
      implicit none                       ! force delcaration of all variables
      include 'seidim.inc'                ! dimensions for rea block
      include 'rea.inc'                   ! parameter common bliock

      character*80 data(5000)             ! s-file with data in text array
      character*80 err_text               ! error text
      character*80 text
      character*80 infile                 ! input file
      character*3  agency                 ! agency for mag and hypo

      logical all                         ! true: read all data, false: headers
      logical eof                         ! true: end of file
      logical head                        ! true if header info
      logical read_head                   ! true if header should be read
      logical dup                         ! true if duplicate header
      character*1 answer
      integer code                        ! error return code
      real min                            ! minute of degree
      integer nevent                      ! number of events in file
      logical arc                         ! true if arc line
      integer i,k                         ! counters
c
c   open output file
c
       open(2,file='hinor.out',status='unknown')
    
c
c   get input file name, check if exist
c

 9    continue
      write(6,*) 'Give input file'
      read(5,'(a)') infile
      open(1,file=infile,status='old',err=10)
      goto 11
 10   continue
      write(6,*)' No such input file'
      goto 9
 11   continue
c
c   agency
c
      write(6,*) 'Agency, 3 letters'
      read(5,'(a3)') agency
      write(6,*) 'Duplicate header line(y=default,n)'
      read(5,'(a)') answer
      dup=.true.
      if(answer.eq.'n') dup=.false.
c
c   archive line
c
      write(6,*) 'Add archive line (y/n=default)'
      read(5,'(a)') answer 
      arc=.true.
      if(answer.eq.' '.or.answer.eq.'n') arc=.true.
      all=.true.                  ! read all parameters
      eof=.false.
      nevent=0                    ! initialize counter
      read_head=.true.            ! header should be read first time

c
c-----------------------------------------------------------------
c  Loop to read events start here
c-----------------------------------------------------------------
c


c
c   new event
c
  50  continue

c
c   clear variables
c
      rea_nphase=0      ! count number of phase lines
      rea_nhyp=0        ! count number of hypocenters
      rea_nwav=0        ! --------------- waveform files
      rea_ncomment=0    ! --------------- comment lines
      rea_nmacro=0      ! --------------- macroseismic data
      rea_nfault=0      ! --------------- fault plane solutions
      rea_nspec=0       ! --------------- spectral solutions
      rea_nmag=0        ! --------------- magnitudes
      rea_id_line=' '   ! no id line

      k=0               ! phase counter
      call rea_hyp_clear(1)
      call rea_hyp_clear(2)
      do i=1,500
         call rea_phase_clear(i)
      enddo
      if(arc) then
         rea_nwav=1
         rea_wav(1)=' '
         rea_wav(1)(2:4)='ARC'
         rea_wav(1)(6:6)='P'
         rea_wav(1)(80:80)='6'
      else
         rea_nwav=0
      endif  
      rea_locality=' '   
c
c  read one line
c
 20   continue
      read(1,'(a)',end=21) text
      goto 22
 21   continue
      eof=.true.
      goto 30
 22   continue
c
c   check if header line
c
c      if(text(55:70).eq.' ') then  ! header of next event
       read(text(1:4),'(i4)',err=300) i   ! header if no error
         if(read_head) then
            head=.false.                   ! might not contain location
c
c  check if origin time and location
c
            if(text(19:19).eq.'N'.or.text(19:19).eq.'S') then
               head=.true.
               read(text(1:4),'(i4)') hyp_year(1)
               read(text(5:6),'(i2)') hyp_month(1)
               read(text(7:8),'(i2)') hyp_day(1)
               read(text(9:10),'(i2)') hyp_hour(1)
               read(text(11:12),'(i2)') hyp_min(1)
               read(text(13:16),'(f4.2)') hyp_sec(1)
               read(text(17:18),'(i2)') i
               read(text(20:23),'(f4.2)') min
               hyp_lat(1)=i+min/60.0
               if(text(19:19).eq.'S') hyp_lat(1)=-hyp_lat(1)
c  lon
               read(text(24:26),'(i3)') i
               read(text(28:31),'(f4.2)') min
               hyp_lon(1)=i+min/60.0
               if(text(27:27).eq.'W') hyp_lon(1)=-hyp_lon(1)
               read(text(32:37),'(f5.2)') hyp_depth(1)
               read(text(37:39),'(f3.2)')hyp_mag(1,1)
c
c   since this is amplitude based magnitude, we assume it is L
c
               hyp_mag_type(1,1)='L'
               rea_nmag=1
               read(text(49:52),'(f4.2)') hyp_rms(1)
               hyp_mag_agency(1,1)(1:3)=agency
               hyp_agency(1)(1:3)=agency

             endif
             goto 20     ! read next line which should be a phase           
           else
             backspace 1    ! this header for next event           
             goto 30        ! new event so go to write out current event
           endif
c        endif
 300    continue
        read_head=.false.      
c
c   assume one more phase
c
      k=k+1
c
c   read phases on phase line
c
c
c   start with assumed p, could be blank, then s
c
        
      rea_stat(k)=text(1:5)
      rea_co(k)(1:1)=text(10:10)
      rea_co(k)(2:2)=text(12:12)
      rea_onset(k)=text(14:14)
      rea_polarity(k)=text(16:16)
      if(rea_polarity(k).eq.'U') rea_polarity(k)='C'
      if(text(15:15).ne.' ') then
         rea_phase(k)=text(15:15) ! is p
         rea_weight_in(k)=text(17:17)
         read(text(30:34),'(f5.2)') rea_sec(k)
      endif
c
c   date and time same for p and s except sec
c
      read(text(18:29), '(i4,4i2)') rea_year(k),rea_month(k),
     *rea_day(k),rea_hour(k),rea_min(k)
      read(text(30:34),'(f5.2)') rea_sec(k)                   
c
c  now assumed s, has same date and time except secs
c
c
c   there is a second phase
c
      if(text(48:48).ne.' ') then

c
c   there was a p so a new phase, rest same as for p
c
         if(text(15:15).ne.' ')then
            k=k+1  ! new phase
            rea_stat(k)=rea_stat(k-1)
            rea_co(k)=rea_co(k-1)
            rea_year(k)=rea_year(k-1)
            rea_month(k)=rea_month(k-1)
            rea_day(k)=rea_day(k-1)
            rea_hour(k)=rea_hour(k-1)
            rea_min(k)=rea_min(k-1)
          endif   
c
c   only s-phase
c
         read(text(42:46),'(f5.2)') rea_sec(k)
         rea_onset(k)=text(47:47)
         rea_phase(k)=text(48:48)
         rea_weight_in(k)=text(50:50)
      endif
c
c   go for next phase
c
      goto 20
c
c  end of one event
c
 30   continue
      rea_nphase=k       
      nevent=nevent+1               ! count events

c
c   write the whole first header line
c
      write(6,'(a)') data(1)(1:79)

c
c   write out event
c
      if(.not.head) then
          hyp_year(1)=rea_year(1)
          hyp_month(1)=rea_month(1)
          hyp_day(1)=rea_day(1)
          hyp_hour(1)=rea_hour(1)
          hyp_min(1)=rea_min(1)
          hyp_sec(1)=rea_sec(1)
       endif

       hyp_dist_id(1)='L'
       rea_nhyp=1
       if(dup) then
          hyp_year(2)=hyp_year(1)
          hyp_month(2)=hyp_month(1)
          hyp_day(2)=hyp_day(1)
          hyp_hour(2)=hyp_hour(1)
          hyp_min(2)=hyp_min(1)
          hyp_sec(2)=hyp_sec(1)
          hyp_dist_id(2)=hyp_dist_id(1)
          hyp_lat(2)=hyp_lat(1)
          hyp_lon(2)=hyp_lon(1)
          hyp_depth(2)=hyp_depth(1)
          hyp_agency(2)=hyp_agency(1)
          hyp_rms(2)=hyp_rms(1)
          hyp_mag(1,2)=hyp_mag(1,1)
          hyp_mag_agency(1,2)=hyp_mag_agency(1,1)
          hyp_mag_type(1,2)=hyp_mag_type(1,1)
          rea_nhyp=2
        endif
       call rea_event_out(2,all,data,code)
c
c   get next event if not end of file
c
      read_head=.true.
      if(.not.eof) goto 50
c
c     end of file
c
c
      write(6,*)            ! blank line
      close(2)              ! close output file
      write(6,*) 'Number of events in input file', nevent
      write(6,*) 'Output file name is hinor.out'

      stop
      end
