package getpde;

import java.io.File;
import org.simpleframework.xml.Element;
import org.simpleframework.xml.Root;
import org.simpleframework.xml.Serializer;
import org.simpleframework.xml.core.Persister;

@Root(name = "GetPDEConfiguration")
public class Configuration {

    @Element(name = "seisanTop", required = false)
    public String SEISAN_TOP = ""; //Seisan top, used to set database
    @Element(name = "dbName")
    public String REABASE = "PDE__"; //REA data base to store settings
    @Element(name = "updatePeriod")
    public int updatePeriod = 5; //Update period in minutes
    public boolean running = true; //Boolean to indicate if acquisition is started
    public File configFile;
    @Element
    public String webAddress = "http://neic.usgs.gov/neis/finger/quake.asc";

    public void processCommand(String line) throws Exception {
        //Remove spaces
        line = line.trim();
        //Separate command and value
        String[] parts = line.split("[\\s]+");
        //Process each possible command
        if (line.trim().equals("")) {
            //Show current configuration
            System.out.println("GetPDE Settings:");
            System.out.println(" SEISAN_TOP: " + SEISAN_TOP);
            System.out.println(" REABASE: " + REABASE);
            System.out.println(" Update period: " + updatePeriod + " minutes");
            System.out.println(" Pull address: " + webAddress);
            System.out.println(" Data pull " + ((running) ? "running" : "stopped"));
            System.out.println("\n\n\n  ? to list commands.");
            return;
        }
        if (line.trim().equals("?")) {
            //TODO: Show possible commands
            System.out.println("GetPDE Commands:");
            System.out.println("Return    : Show current configuration.");
            System.out.println("?         : Show GetPDE commands.");
            System.out.println("SEISAN_TOP <folder> : set SEISAN_TOP folder manually");
            System.out.println("REABASE <name>      : set name of the REA database");
            System.out.println("UPDATEPER <minutes> : set update period in minutes");
            System.out.println("WEBADDRESS <address>: address of the USGS file to pull.");
            System.out.println("START               : start pulling data.");
            System.out.println("STOP                : stop pulling data.");
            return;
        }
        if (parts[0].equalsIgnoreCase(
                "START")) {
            System.out.println("Starting acquisition");
            running = true;
            return;
        }
        if (parts[0].equalsIgnoreCase(
                "STOP")) {
            System.out.println("Stopping acquisition");
            running = false;
            return;
        }
        if (parts.length
                < 2) {
            throw new Exception("Invalid sintax.");
        }
        if (parts[0].equalsIgnoreCase(
                "SEISAN_TOP")) {
            //Change seisan top
            SEISAN_TOP = parts[1];
            saveConfig();
            return;
        }
        if (parts[0].equalsIgnoreCase(
                "REABASE")) {
            //Change rea base
            if (parts[1].length() > 5) {
                throw new Exception("REABASE name too large");
            }
            REABASE = parts[1];
            for (int i = REABASE.length(); i < 5; i++) {
                REABASE += "_";
            }
            saveConfig();
            return;
        }
        if (parts[0].equalsIgnoreCase(
                "UPDATEPER")) {
            //Change updating period
            try {
                updatePeriod = Integer.valueOf(parts[1]);
                saveConfig();
            } catch (Exception e) {
                throw new Exception("Invalid update period.");
            }
            return;
        }
        if (parts[0].equalsIgnoreCase(
                "WEBADDRESS")) {
            //Change web address
            webAddress = parts[1];
            saveConfig();
            return;
        }
        //Command is unknown
        throw new Exception(
                "Unknown command.");
    }

    private void saveConfig() throws Exception {
        Serializer serializer = new Persister();
        serializer.write(this, configFile);
    }

    public static Configuration loadConfig(File configfile) throws Exception {
        Configuration config = new Configuration();
        Serializer serializer = new Persister();
        config = serializer.read(Configuration.class, configfile);
        config.configFile = configfile;
        if (config.SEISAN_TOP.isEmpty()) {
            config.SEISAN_TOP = System.getenv("SEISAN_TOP");
        }
        return config;
    }

}
